@@ -8,6 +8,7 @@ use std::{
     fmt,
     hash::Hash,
     ops::{Add, AddAssign},
+    ptr,
 };
 
 /// Compiled AST (abstract syntax tree) of a Rhai script.
@@ -847,6 +848,7 @@ impl AsRef<crate::Shared<crate::Module>> for AST {
 /// _(internals)_ An [`AST`] node, consisting of either an [`Expr`] or a [`Stmt`].
 /// Exported under the `internals` feature only.
 #[derive(Debug, Clone, Copy, Hash)]
+#[non_exhaustive]
 pub enum ASTNode<'a> {
     /// A statement ([`Stmt`]).
     Stmt(&'a Stmt),
@@ -870,8 +872,8 @@ impl PartialEq for ASTNode<'_> {
     #[inline(always)]
     fn eq(&self, other: &Self) -> bool {
         match (self, other) {
-            (Self::Stmt(x), Self::Stmt(y)) => std::ptr::eq(*x, *y),
-            (Self::Expr(x), Self::Expr(y)) => std::ptr::eq(*x, *y),
+            (Self::Stmt(x), Self::Stmt(y)) => ptr::eq(*x, *y),
+            (Self::Expr(x), Self::Expr(y)) => ptr::eq(*x, *y),
             _ => false,
         }
     }
