@@ -60,15 +60,13 @@ fn print_current_source(
     lines: &[String],
     window: (usize, usize),
 ) {
-    let current_source = &mut *context
-        .global_runtime_state_mut()
-        .debugger
-        .state_mut()
-        .write_lock::<ImmutableString>()
-        .unwrap();
+    let current_source = &mut *context.tag_mut().write_lock::<ImmutableString>().unwrap();
     let src = source.unwrap_or("");
     if src != current_source {
-        println!(">>> Source => {}", source.unwrap_or("main script"));
+        println!(
+            "\x1b[34m>>> Source => {}\x1b[39m",
+            source.unwrap_or("main script")
+        );
         *current_source = src.into();
     }
     if !src.is_empty() {
@@ -118,8 +116,10 @@ fn print_debug_help() {
     println!("help, h                => print this help");
     println!("quit, q, exit, kill    => quit");
     println!("scope                  => print the scope");
+    println!("operations             => print the total operations performed");
+    println!("source                 => print the current source");
     println!("print, p               => print all variables de-duplicated");
-    println!("print/p this           => print the 'this' pointer");
+    println!("print/p this           => print the `this` pointer");
     println!("print/p <variable>     => print the current value of a variable");
     #[cfg(not(feature = "no_module"))]
     println!("imports                => print all imported modules");
@@ -154,43 +154,6 @@ fn print_debug_help() {
     println!();
 }
 
-/// Display the current scope.
-fn print_scope(scope: &Scope, dedup: bool) {
-    let flattened_clone;
-    let scope = if dedup {
-        flattened_clone = scope.clone_visible();
-        &flattened_clone
-    } else {
-        scope
-    };
-
-    for (i, (name, constant, value)) in scope.iter_raw().enumerate() {
-        #[cfg(not(feature = "no_closure"))]
-        let value_is_shared = if value.is_shared() { " (shared)" } else { "" };
-        #[cfg(feature = "no_closure")]
-        let value_is_shared = "";
-
-        if dedup {
-            println!(
-                "{}{}{} = {:?}",
-                if constant { "const " } else { "" },
-                name,
-                value_is_shared,
-                *value.read_lock::<Dynamic>().unwrap(),
-            );
-        } else {
-            println!(
-                "[{}] {}{}{} = {:?}",
-                i + 1,
-                if constant { "const " } else { "" },
-                name,
-                value_is_shared,
-                *value.read_lock::<Dynamic>().unwrap(),
-            );
-        }
-    }
-}
-
 // Load script to debug.
 fn load_script(engine: &Engine) -> (rhai::AST, String) {
     if let Some(filename) = env::args().skip(1).next() {
@@ -262,7 +225,7 @@ fn load_script(engine: &Engine) -> (rhai::AST, String) {
 
 // Main callback for debugging.
 fn debug_callback(
-    context: &mut rhai::EvalContext,
+    mut context: rhai::EvalContext,
     event: DebuggerEvent,
     node: rhai::ASTNode,
     source: Option<&str>,
@@ -271,6 +234,8 @@ fn debug_callback(
 ) -> Result<DebuggerCommand, Box<EvalAltResult>> {
     // Check event
     match event {
+        DebuggerEvent::Start => println!("\x1b[32m! Script start\x1b[39m"),
+        DebuggerEvent::End => println!("\x1b[31m! Script end\x1b[39m"),
         DebuggerEvent::Step => (),
         DebuggerEvent::BreakPoint(n) => {
             match context.global_runtime_state().debugger.break_points()[n] {
@@ -284,6 +249,7 @@ fn debug_callback(
                 BreakPoint::AtProperty { ref name, .. } => {
                     println!("! Property {} accessed.", name)
                 }
+                _ => unreachable!(),
             }
         }
         DebuggerEvent::FunctionExitWithValue(r) => {
@@ -312,10 +278,11 @@ fn debug_callback(
                 err
             )
         }
+        _ => unreachable!(),
     }
 
     // Print current source line
-    print_current_source(context, source, pos, lines, (0, 0));
+    print_current_source(&mut context, source, pos, lines, (0, 0));
 
     // Read stdin for commands
     let mut input = String::new();
@@ -350,14 +317,20 @@ fn debug_callback(
                     }
                     println!();
                 }
-                ["list" | "l"] => print_current_source(context, source, pos, &lines, (3, 6)),
+                ["operations"] => {
+                    println!("{}", context.global_runtime_state().num_operations)
+                }
+                ["source"] => {
+                    println!("{}", context.global_runtime_state().source().unwrap_or(""))
+                }
+                ["list" | "l"] => print_current_source(&mut context, source, pos, &lines, (3, 6)),
                 ["list" | "l", n] if n.parse::<usize>().is_ok() => {
                     let num = n.parse::<usize>().unwrap();
                     if num <= 0 || num > lines.len() {
                         eprintln!("\x1b[31mInvalid line: {}\x1b[39m", num);
                     } else {
                         let pos = Position::new(num as u16, 0);
-                        print_current_source(context, source, pos, &lines, (3, 6));
+                        print_current_source(&mut context, source, pos, &lines, (3, 6));
                     }
                 }
                 ["continue" | "c"] => break Ok(DebuggerCommand::Continue),
@@ -365,12 +338,12 @@ fn debug_callback(
                 [] | ["step" | "s"] => break Ok(DebuggerCommand::StepInto),
                 ["over" | "o"] => break Ok(DebuggerCommand::StepOver),
                 ["next" | "n"] => break Ok(DebuggerCommand::Next),
-                ["scope"] => print_scope(context.scope(), false),
+                ["scope"] => println!("{}", context.scope()),
                 ["print" | "p", "this"] => {
                     if let Some(value) = context.this_ptr() {
                         println!("=> {:?}", value);
                     } else {
-                        println!("'this' pointer is unbound.");
+                        println!("`this` pointer is unbound.");
                     }
                 }
                 ["print" | "p", var_name] => {
@@ -381,7 +354,7 @@ fn debug_callback(
                     }
                 }
                 ["print" | "p"] => {
-                    print_scope(context.scope(), true);
+                    println!("{}", context.scope().clone_visible());
                     if let Some(value) = context.this_ptr() {
                         println!("this = {:?}", value);
                     }
@@ -669,4 +642,6 @@ fn main() {
             }
         }
     }
+
+    println!("Script terminated. Bye!");
 }
