@@ -1,7 +1,7 @@
 //! Types to support chaining operations (i.e. indexing and dotting).
 #![cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
 
-use super::{EvalState, GlobalRuntimeState, Target};
+use super::{Caches, GlobalRuntimeState, Target};
 use crate::ast::{ASTFlags, Expr, OpAssignment};
 use crate::types::dynamic::Union;
 use crate::{Dynamic, Engine, Module, Position, RhaiResult, RhaiResultOf, Scope, StaticVec, ERR};
@@ -122,7 +122,7 @@ impl Engine {
     fn eval_dot_index_chain_helper(
         &self,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         target: &mut Target,
@@ -130,10 +130,10 @@ impl Engine {
         _parent: &Expr,
         rhs: &Expr,
         _parent_options: ASTFlags,
-        idx_values: &mut StaticVec<super::ChainArgument>,
+        idx_values: &mut StaticVec<ChainArgument>,
         chain_type: ChainType,
         level: usize,
-        new_val: Option<((Dynamic, Position), (Option<OpAssignment>, Position))>,
+        new_val: Option<(Dynamic, OpAssignment)>,
     ) -> RhaiResultOf<(Dynamic, bool)> {
         let is_ref_mut = target.is_ref();
 
@@ -155,7 +155,7 @@ impl Engine {
                         if !_parent_options.contains(ASTFlags::BREAK) =>
                     {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, _parent, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, _parent, level)?;
 
                         let mut idx_val_for_setter = idx_val.clone();
                         let idx_pos = x.lhs.start_position();
@@ -163,14 +163,14 @@ impl Engine {
 
                         let (try_setter, result) = {
                             let mut obj = self.get_indexed_mut(
-                                global, state, lib, target, idx_val, idx_pos, false, true, level,
+                                global, caches, lib, target, idx_val, idx_pos, false, true, level,
                             )?;
                             let is_obj_temp_val = obj.is_temp_value();
                             let obj_ptr = &mut obj;
 
                             match self.eval_dot_index_chain_helper(
-                                global, state, lib, this_ptr, obj_ptr, root, rhs, &x.rhs, *options,
-                                idx_values, rhs_chain, level, new_val,
+                                global, caches, lib, this_ptr, obj_ptr, root, rhs, &x.rhs,
+                                *options, idx_values, rhs_chain, level, new_val,
                             ) {
                                 Ok((result, true)) if is_obj_temp_val => {
                                     (Some(obj.take_or_clone()), (result, true))
@@ -185,7 +185,7 @@ impl Engine {
                             let idx = &mut idx_val_for_setter;
                             let new_val = &mut new_val;
                             self.call_indexer_set(
-                                global, state, lib, target, idx, new_val, is_ref_mut, level,
+                                global, caches, lib, target, idx, new_val, is_ref_mut, level,
                             )
                             .or_else(|e| match *e {
                                 ERR::ErrorIndexingType(..) => Ok((Dynamic::UNIT, false)),
@@ -198,26 +198,24 @@ impl Engine {
                     // xxx[rhs] op= new_val
                     _ if new_val.is_some() => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, _parent, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, _parent, level)?;
 
-                        let ((new_val, new_pos), (op_info, op_pos)) = new_val.expect("`Some`");
-                        let mut idx_val_for_setter = idx_val.clone();
+                        let (new_val, op_info) = new_val.expect("`Some`");
+                        let mut idx_val2 = idx_val.clone();
 
                         let try_setter = match self.get_indexed_mut(
-                            global, state, lib, target, idx_val, pos, true, false, level,
+                            global, caches, lib, target, idx_val, pos, true, false, level,
                         ) {
-                            // Indexed value is a reference - update directly
+                            // Indexed value is not a temp value - update directly
                             Ok(ref mut obj_ptr) => {
                                 self.eval_op_assignment(
-                                    global, state, lib, op_info, op_pos, obj_ptr, root, new_val,
-                                    level,
-                                )
-                                .map_err(|err| err.fill_position(new_pos))?;
+                                    global, caches, lib, op_info, obj_ptr, root, new_val, level,
+                                )?;
                                 #[cfg(not(feature = "unchecked"))]
-                                self.check_data_size(obj_ptr, new_pos)?;
+                                self.check_data_size(obj_ptr, op_info.pos)?;
                                 None
                             }
-                            // Can't index - try to call an index setter
+                            // Indexed value cannot be referenced - use indexer
                             #[cfg(not(feature = "no_index"))]
                             Err(err) if matches!(*err, ERR::ErrorIndexingType(..)) => Some(new_val),
                             // Any other error
@@ -225,11 +223,32 @@ impl Engine {
                         };
 
                         if let Some(mut new_val) = try_setter {
+                            let idx = &mut idx_val2;
+
+                            // Is this an op-assignment?
+                            if op_info.is_op_assignment() {
+                                let idx = &mut idx.clone();
+                                // Call the index getter to get the current value
+                                if let Ok(val) =
+                                    self.call_indexer_get(global, caches, lib, target, idx, level)
+                                {
+                                    let mut res = val.into();
+                                    // Run the op-assignment
+                                    self.eval_op_assignment(
+                                        global, caches, lib, op_info, &mut res, root, new_val,
+                                        level,
+                                    )?;
+                                    // Replace new value
+                                    new_val = res.take_or_clone();
+                                    #[cfg(not(feature = "unchecked"))]
+                                    self.check_data_size(&new_val, op_info.pos)?;
+                                }
+                            }
+
                             // Try to call index setter
-                            let idx = &mut idx_val_for_setter;
                             let new_val = &mut new_val;
                             self.call_indexer_set(
-                                global, state, lib, target, idx, new_val, is_ref_mut, level,
+                                global, caches, lib, target, idx, new_val, is_ref_mut, level,
                             )?;
                         }
 
@@ -238,10 +257,10 @@ impl Engine {
                     // xxx[rhs]
                     _ => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, _parent, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, _parent, level)?;
 
                         self.get_indexed_mut(
-                            global, state, lib, target, idx_val, pos, false, true, level,
+                            global, caches, lib, target, idx_val, pos, false, true, level,
                         )
                         .map(|v| (v.take_or_clone(), false))
                     }
@@ -257,12 +276,11 @@ impl Engine {
                         let call_args = &mut idx_val.into_fn_call_args();
 
                         #[cfg(feature = "debugging")]
-                        let reset_debugger = self.run_debugger_with_reset(
-                            scope, global, state, lib, this_ptr, rhs, level,
-                        )?;
+                        let reset_debugger =
+                            self.run_debugger_with_reset(scope, global, lib, this_ptr, rhs, level)?;
 
                         let result = self.make_method_call(
-                            global, state, lib, name, *hashes, target, call_args, *pos, level,
+                            global, caches, lib, name, *hashes, target, call_args, *pos, level,
                         );
 
                         #[cfg(feature = "debugging")]
@@ -281,58 +299,57 @@ impl Engine {
                     // {xxx:map}.id op= ???
                     Expr::Property(x, pos) if target.is::<crate::Map>() && new_val.is_some() => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, rhs, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, rhs, level)?;
 
                         let index = x.2.clone().into();
-                        let ((new_val, new_pos), (op_info, op_pos)) = new_val.expect("`Some`");
+                        let (new_val, op_info) = new_val.expect("`Some`");
                         {
                             let val_target = &mut self.get_indexed_mut(
-                                global, state, lib, target, index, *pos, true, false, level,
+                                global, caches, lib, target, index, *pos, true, false, level,
                             )?;
                             self.eval_op_assignment(
-                                global, state, lib, op_info, op_pos, val_target, root, new_val,
-                                level,
-                            )
-                            .map_err(|err| err.fill_position(new_pos))?;
+                                global, caches, lib, op_info, val_target, root, new_val, level,
+                            )?;
                         }
                         #[cfg(not(feature = "unchecked"))]
-                        self.check_data_size(target.source(), new_pos)?;
+                        self.check_data_size(target.source(), op_info.pos)?;
                         Ok((Dynamic::UNIT, true))
                     }
                     // {xxx:map}.id
                     Expr::Property(x, pos) if target.is::<crate::Map>() => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, rhs, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, rhs, level)?;
 
                         let index = x.2.clone().into();
                         let val = self.get_indexed_mut(
-                            global, state, lib, target, index, *pos, false, false, level,
+                            global, caches, lib, target, index, *pos, false, false, level,
                         )?;
                         Ok((val.take_or_clone(), false))
                     }
                     // xxx.id op= ???
                     Expr::Property(x, pos) if new_val.is_some() => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, rhs, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, rhs, level)?;
 
                         let ((getter, hash_get), (setter, hash_set), name) = x.as_ref();
-                        let ((mut new_val, new_pos), (op_info, op_pos)) = new_val.expect("`Some`");
+                        let (mut new_val, op_info) = new_val.expect("`Some`");
 
-                        if op_info.is_some() {
+                        if op_info.is_op_assignment() {
                             let hash = crate::ast::FnCallHashes::from_native(*hash_get);
                             let args = &mut [target.as_mut()];
                             let (mut orig_val, ..) = self
                                 .exec_fn_call(
-                                    None, global, state, lib, getter, hash, args, is_ref_mut, true,
-                                    *pos, level,
+                                    None, global, caches, lib, getter, hash, args, is_ref_mut,
+                                    true, *pos, level,
                                 )
                                 .or_else(|err| match *err {
                                     // Try an indexer if property does not exist
                                     ERR::ErrorDotExpr(..) => {
                                         let mut prop = name.into();
                                         self.call_indexer_get(
-                                            global, state, lib, target, &mut prop, level,
+                                            global, caches, lib, target, &mut prop, level,
                                         )
+                                        .map(|r| (r, false))
                                         .map_err(|e| {
                                             match *e {
                                                 ERR::ErrorIndexingType(..) => err,
@@ -347,10 +364,8 @@ impl Engine {
                                 let orig_val = &mut (&mut orig_val).into();
 
                                 self.eval_op_assignment(
-                                    global, state, lib, op_info, op_pos, orig_val, root, new_val,
-                                    level,
-                                )
-                                .map_err(|err| err.fill_position(new_pos))?;
+                                    global, caches, lib, op_info, orig_val, root, new_val, level,
+                                )?;
                             }
 
                             new_val = orig_val;
@@ -359,7 +374,7 @@ impl Engine {
                         let hash = crate::ast::FnCallHashes::from_native(*hash_set);
                         let args = &mut [target.as_mut(), &mut new_val];
                         self.exec_fn_call(
-                            None, global, state, lib, setter, hash, args, is_ref_mut, true, *pos,
+                            None, global, caches, lib, setter, hash, args, is_ref_mut, true, *pos,
                             level,
                         )
                         .or_else(|err| match *err {
@@ -368,7 +383,7 @@ impl Engine {
                                 let idx = &mut name.into();
                                 let new_val = &mut new_val;
                                 self.call_indexer_set(
-                                    global, state, lib, target, idx, new_val, is_ref_mut, level,
+                                    global, caches, lib, target, idx, new_val, is_ref_mut, level,
                                 )
                                 .map_err(|e| match *e {
                                     ERR::ErrorIndexingType(..) => err,
@@ -381,13 +396,13 @@ impl Engine {
                     // xxx.id
                     Expr::Property(x, pos) => {
                         #[cfg(feature = "debugging")]
-                        self.run_debugger(scope, global, state, lib, this_ptr, rhs, level)?;
+                        self.run_debugger(scope, global, lib, this_ptr, rhs, level)?;
 
                         let ((getter, hash_get), _, name) = x.as_ref();
                         let hash = crate::ast::FnCallHashes::from_native(*hash_get);
                         let args = &mut [target.as_mut()];
                         self.exec_fn_call(
-                            None, global, state, lib, getter, hash, args, is_ref_mut, true, *pos,
+                            None, global, caches, lib, getter, hash, args, is_ref_mut, true, *pos,
                             level,
                         )
                         .map_or_else(
@@ -396,8 +411,9 @@ impl Engine {
                                 ERR::ErrorDotExpr(..) => {
                                     let mut prop = name.into();
                                     self.call_indexer_get(
-                                        global, state, lib, target, &mut prop, level,
+                                        global, caches, lib, target, &mut prop, level,
                                     )
+                                    .map(|r| (r, false))
                                     .map_err(|e| match *e {
                                         ERR::ErrorIndexingType(..) => err,
                                         _ => e,
@@ -418,13 +434,11 @@ impl Engine {
                         let val_target = &mut match x.lhs {
                             Expr::Property(ref p, pos) => {
                                 #[cfg(feature = "debugging")]
-                                self.run_debugger(
-                                    scope, global, state, lib, this_ptr, _node, level,
-                                )?;
+                                self.run_debugger(scope, global, lib, this_ptr, _node, level)?;
 
                                 let index = p.2.clone().into();
                                 self.get_indexed_mut(
-                                    global, state, lib, target, index, pos, false, true, level,
+                                    global, caches, lib, target, index, pos, false, true, level,
                                 )?
                             }
                             // {xxx:map}.fn_name(arg_expr_list)[expr] | {xxx:map}.fn_name(arg_expr_list).expr
@@ -434,11 +448,11 @@ impl Engine {
 
                                 #[cfg(feature = "debugging")]
                                 let reset_debugger = self.run_debugger_with_reset(
-                                    scope, global, state, lib, this_ptr, _node, level,
+                                    scope, global, lib, this_ptr, _node, level,
                                 )?;
 
                                 let result = self.make_method_call(
-                                    global, state, lib, name, *hashes, target, call_args, pos,
+                                    global, caches, lib, name, *hashes, target, call_args, pos,
                                     level,
                                 );
 
@@ -457,7 +471,7 @@ impl Engine {
                         let rhs_chain = rhs.into();
 
                         self.eval_dot_index_chain_helper(
-                            global, state, lib, this_ptr, val_target, root, rhs, &x.rhs, *options,
+                            global, caches, lib, this_ptr, val_target, root, rhs, &x.rhs, *options,
                             idx_values, rhs_chain, level, new_val,
                         )
                         .map_err(|err| err.fill_position(*x_pos))
@@ -470,9 +484,7 @@ impl Engine {
                             // xxx.prop[expr] | xxx.prop.expr
                             Expr::Property(ref p, pos) => {
                                 #[cfg(feature = "debugging")]
-                                self.run_debugger(
-                                    scope, global, state, lib, this_ptr, _node, level,
-                                )?;
+                                self.run_debugger(scope, global, lib, this_ptr, _node, level)?;
 
                                 let ((getter, hash_get), (setter, hash_set), name) = p.as_ref();
                                 let rhs_chain = rhs.into();
@@ -484,7 +496,7 @@ impl Engine {
                                 // Assume getters are always pure
                                 let (mut val, ..) = self
                                     .exec_fn_call(
-                                        None, global, state, lib, getter, hash_get, args,
+                                        None, global, caches, lib, getter, hash_get, args,
                                         is_ref_mut, true, pos, level,
                                     )
                                     .or_else(|err| match *err {
@@ -492,8 +504,9 @@ impl Engine {
                                         ERR::ErrorDotExpr(..) => {
                                             let mut prop = name.into();
                                             self.call_indexer_get(
-                                                global, state, lib, target, &mut prop, level,
+                                                global, caches, lib, target, &mut prop, level,
                                             )
+                                            .map(|r| (r, false))
                                             .map_err(
                                                 |e| match *e {
                                                     ERR::ErrorIndexingType(..) => err,
@@ -508,7 +521,7 @@ impl Engine {
 
                                 let (result, may_be_changed) = self
                                     .eval_dot_index_chain_helper(
-                                        global, state, lib, this_ptr, val, root, rhs, &x.rhs,
+                                        global, caches, lib, this_ptr, val, root, rhs, &x.rhs,
                                         *options, idx_values, rhs_chain, level, new_val,
                                     )
                                     .map_err(|err| err.fill_position(*x_pos))?;
@@ -519,7 +532,7 @@ impl Engine {
                                     let mut arg_values = [target.as_mut(), val.as_mut()];
                                     let args = &mut arg_values;
                                     self.exec_fn_call(
-                                        None, global, state, lib, setter, hash_set, args,
+                                        None, global, caches, lib, setter, hash_set, args,
                                         is_ref_mut, true, pos, level,
                                     )
                                     .or_else(
@@ -529,7 +542,7 @@ impl Engine {
                                                 let idx = &mut name.into();
                                                 let new_val = val;
                                                 self.call_indexer_set(
-                                                    global, state, lib, target, idx, new_val,
+                                                    global, caches, lib, target, idx, new_val,
                                                     is_ref_mut, level,
                                                 )
                                                 .or_else(|e| match *e {
@@ -556,11 +569,11 @@ impl Engine {
 
                                 #[cfg(feature = "debugging")]
                                 let reset_debugger = self.run_debugger_with_reset(
-                                    scope, global, state, lib, this_ptr, _node, level,
+                                    scope, global, lib, this_ptr, _node, level,
                                 )?;
 
                                 let result = self.make_method_call(
-                                    global, state, lib, name, *hashes, target, args, pos, level,
+                                    global, caches, lib, name, *hashes, target, args, pos, level,
                                 );
 
                                 #[cfg(feature = "debugging")]
@@ -570,8 +583,8 @@ impl Engine {
                                 let val = &mut val.into();
 
                                 self.eval_dot_index_chain_helper(
-                                    global, state, lib, this_ptr, val, root, rhs, &x.rhs, *options,
-                                    idx_values, rhs_chain, level, new_val,
+                                    global, caches, lib, this_ptr, val, root, rhs, &x.rhs,
+                                    *options, idx_values, rhs_chain, level, new_val,
                                 )
                                 .map_err(|err| err.fill_position(pos))
                             }
@@ -595,12 +608,12 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         expr: &Expr,
         level: usize,
-        new_val: Option<((Dynamic, Position), (Option<OpAssignment>, Position))>,
+        new_val: Option<(Dynamic, OpAssignment)>,
     ) -> RhaiResult {
         let (crate::ast::BinaryExpr { lhs, rhs }, chain_type, options, op_pos) = match expr {
             #[cfg(not(feature = "no_index"))]
@@ -613,7 +626,7 @@ impl Engine {
         let idx_values = &mut StaticVec::new_const();
 
         self.eval_dot_index_chain_arguments(
-            scope, global, state, lib, this_ptr, rhs, options, chain_type, idx_values, 0, level,
+            scope, global, caches, lib, this_ptr, rhs, options, chain_type, idx_values, 0, level,
         )?;
 
         let is_assignment = new_val.is_some();
@@ -622,19 +635,19 @@ impl Engine {
             // id.??? or id[???]
             Expr::Variable(x, .., var_pos) => {
                 #[cfg(feature = "debugging")]
-                self.run_debugger(scope, global, state, lib, this_ptr, lhs, level)?;
+                self.run_debugger(scope, global, lib, this_ptr, lhs, level)?;
 
                 #[cfg(not(feature = "unchecked"))]
                 self.inc_operations(&mut global.num_operations, *var_pos)?;
 
                 let (mut target, ..) =
-                    self.search_namespace(scope, global, state, lib, this_ptr, lhs, level)?;
+                    self.search_namespace(scope, global, lib, this_ptr, lhs, level)?;
 
                 let obj_ptr = &mut target;
                 let root = (x.3.as_str(), *var_pos);
 
                 self.eval_dot_index_chain_helper(
-                    global, state, lib, &mut None, obj_ptr, root, expr, rhs, options, idx_values,
+                    global, caches, lib, &mut None, obj_ptr, root, expr, rhs, options, idx_values,
                     chain_type, level, new_val,
                 )
                 .map(|(v, ..)| v)
@@ -644,11 +657,11 @@ impl Engine {
             _ if is_assignment => unreachable!("cannot assign to an expression"),
             // {expr}.??? or {expr}[???]
             expr => {
-                let value = self.eval_expr(scope, global, state, lib, this_ptr, expr, level)?;
+                let value = self.eval_expr(scope, global, caches, lib, this_ptr, expr, level)?;
                 let obj_ptr = &mut value.into();
                 let root = ("", expr.start_position());
                 self.eval_dot_index_chain_helper(
-                    global, state, lib, this_ptr, obj_ptr, root, expr, rhs, options, idx_values,
+                    global, caches, lib, this_ptr, obj_ptr, root, expr, rhs, options, idx_values,
                     chain_type, level, new_val,
                 )
                 .map(|(v, ..)| if is_assignment { Dynamic::UNIT } else { v })
@@ -664,13 +677,13 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         expr: &Expr,
         parent_options: ASTFlags,
         _parent_chain_type: ChainType,
-        idx_values: &mut StaticVec<super::ChainArgument>,
+        idx_values: &mut StaticVec<ChainArgument>,
         size: usize,
         level: usize,
     ) -> RhaiResultOf<()> {
@@ -688,7 +701,7 @@ impl Engine {
                     (crate::FnArgsVec::with_capacity(args.len()), Position::NONE),
                     |(mut values, mut pos), expr| {
                         let (value, arg_pos) =
-                            self.get_arg_value(scope, global, state, lib, this_ptr, expr, level)?;
+                            self.get_arg_value(scope, global, caches, lib, this_ptr, expr, level)?;
                         if values.is_empty() {
                             pos = arg_pos;
                         }
@@ -697,7 +710,7 @@ impl Engine {
                     },
                 )?;
 
-                idx_values.push(super::ChainArgument::from_fn_call_args(values, pos));
+                idx_values.push(ChainArgument::from_fn_call_args(values, pos));
             }
             #[cfg(not(feature = "no_object"))]
             Expr::MethodCall(..) if _parent_chain_type == ChainType::Dotting => {
@@ -706,7 +719,7 @@ impl Engine {
 
             #[cfg(not(feature = "no_object"))]
             Expr::Property(.., pos) if _parent_chain_type == ChainType::Dotting => {
-                idx_values.push(super::ChainArgument::Property(*pos))
+                idx_values.push(ChainArgument::Property(*pos))
             }
             Expr::Property(..) => unreachable!("unexpected Expr::Property for indexing"),
 
@@ -719,7 +732,7 @@ impl Engine {
                 let lhs_arg_val = match lhs {
                     #[cfg(not(feature = "no_object"))]
                     Expr::Property(.., pos) if _parent_chain_type == ChainType::Dotting => {
-                        super::ChainArgument::Property(*pos)
+                        ChainArgument::Property(*pos)
                     }
                     Expr::Property(..) => unreachable!("unexpected Expr::Property for indexing"),
 
@@ -733,7 +746,7 @@ impl Engine {
                             (crate::FnArgsVec::with_capacity(args.len()), Position::NONE),
                             |(mut values, mut pos), expr| {
                                 let (value, arg_pos) = self.get_arg_value(
-                                    scope, global, state, lib, this_ptr, expr, level,
+                                    scope, global, caches, lib, this_ptr, expr, level,
                                 )?;
                                 if values.is_empty() {
                                     pos = arg_pos
@@ -742,7 +755,7 @@ impl Engine {
                                 Ok::<_, crate::RhaiError>((values, pos))
                             },
                         )?;
-                        super::ChainArgument::from_fn_call_args(values, pos)
+                        ChainArgument::from_fn_call_args(values, pos)
                     }
                     #[cfg(not(feature = "no_object"))]
                     Expr::MethodCall(..) if _parent_chain_type == ChainType::Dotting => {
@@ -754,12 +767,9 @@ impl Engine {
                     }
                     #[cfg(not(feature = "no_index"))]
                     _ if _parent_chain_type == ChainType::Indexing => self
-                        .eval_expr(scope, global, state, lib, this_ptr, lhs, level)
+                        .eval_expr(scope, global, caches, lib, this_ptr, lhs, level)
                         .map(|v| {
-                            super::ChainArgument::from_index_value(
-                                v.flatten(),
-                                lhs.start_position(),
-                            )
+                            ChainArgument::from_index_value(v.flatten(), lhs.start_position())
                         })?,
                     expr => unreachable!("unknown chained expression: {:?}", expr),
                 };
@@ -768,7 +778,7 @@ impl Engine {
                 let chain_type = expr.into();
 
                 self.eval_dot_index_chain_arguments(
-                    scope, global, state, lib, this_ptr, rhs, *options, chain_type, idx_values,
+                    scope, global, caches, lib, this_ptr, rhs, *options, chain_type, idx_values,
                     size, level,
                 )?;
 
@@ -781,10 +791,8 @@ impl Engine {
             }
             #[cfg(not(feature = "no_index"))]
             _ if _parent_chain_type == ChainType::Indexing => idx_values.push(
-                self.eval_expr(scope, global, state, lib, this_ptr, expr, level)
-                    .map(|v| {
-                        super::ChainArgument::from_index_value(v.flatten(), expr.start_position())
-                    })?,
+                self.eval_expr(scope, global, caches, lib, this_ptr, expr, level)
+                    .map(|v| ChainArgument::from_index_value(v.flatten(), expr.start_position()))?,
             ),
             _ => unreachable!("unknown chained expression: {:?}", expr),
         }
@@ -798,20 +806,21 @@ impl Engine {
     fn call_indexer_get(
         &self,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         target: &mut Dynamic,
         idx: &mut Dynamic,
         level: usize,
-    ) -> RhaiResultOf<(Dynamic, bool)> {
+    ) -> RhaiResultOf<Dynamic> {
         let args = &mut [target, idx];
         let hash_get = crate::ast::FnCallHashes::from_native(global.hash_idx_get());
         let fn_name = crate::engine::FN_IDX_GET;
         let pos = Position::NONE;
 
         self.exec_fn_call(
-            None, global, state, lib, fn_name, hash_get, args, true, true, pos, level,
+            None, global, caches, lib, fn_name, hash_get, args, true, true, pos, level,
         )
+        .map(|(r, ..)| r)
     }
 
     /// Call a set indexer.
@@ -820,7 +829,7 @@ impl Engine {
     fn call_indexer_set(
         &self,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         target: &mut Dynamic,
         idx: &mut Dynamic,
@@ -834,7 +843,7 @@ impl Engine {
         let pos = Position::NONE;
 
         self.exec_fn_call(
-            None, global, state, lib, fn_name, hash_set, args, is_ref_mut, true, pos, level,
+            None, global, caches, lib, fn_name, hash_set, args, is_ref_mut, true, pos, level,
         )
     }
 
@@ -843,7 +852,7 @@ impl Engine {
     fn get_indexed_mut<'t>(
         &self,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         target: &'t mut Dynamic,
         mut idx: Dynamic,
@@ -1038,8 +1047,8 @@ impl Engine {
             }
 
             _ if use_indexers => self
-                .call_indexer_get(global, state, lib, target, &mut idx, level)
-                .map(|(v, ..)| v.into()),
+                .call_indexer_get(global, caches, lib, target, &mut idx, level)
+                .map(Into::into),
 
             _ => Err(ERR::ErrorIndexingType(
                 format!(
