@@ -1,12 +1,12 @@
 //! Module defining the debugging interface.
 #![cfg(feature = "debugging")]
 
-use super::{EvalContext, EvalState, GlobalRuntimeState};
+use super::{EvalContext, GlobalRuntimeState};
 use crate::ast::{ASTNode, Expr, Stmt};
 use crate::{Dynamic, Engine, EvalAltResult, Identifier, Module, Position, RhaiResultOf, Scope};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
-use std::{fmt, mem};
+use std::{fmt, iter::repeat, mem};
 
 /// Callback function to initialize the debugger.
 #[cfg(not(feature = "sync"))]
@@ -18,7 +18,7 @@ pub type OnDebuggingInit = dyn Fn() -> Dynamic + Send + Sync;
 /// Callback function for debugging.
 #[cfg(not(feature = "sync"))]
 pub type OnDebuggerCallback = dyn Fn(
-    &mut EvalContext,
+    EvalContext,
     DebuggerEvent,
     ASTNode,
     Option<&str>,
@@ -26,18 +26,13 @@ pub type OnDebuggerCallback = dyn Fn(
 ) -> RhaiResultOf<DebuggerCommand>;
 /// Callback function for debugging.
 #[cfg(feature = "sync")]
-pub type OnDebuggerCallback = dyn Fn(
-        &mut EvalContext,
-        DebuggerEvent,
-        ASTNode,
-        Option<&str>,
-        Position,
-    ) -> RhaiResultOf<DebuggerCommand>
+pub type OnDebuggerCallback = dyn Fn(EvalContext, DebuggerEvent, ASTNode, Option<&str>, Position) -> RhaiResultOf<DebuggerCommand>
     + Send
     + Sync;
 
 /// A command for the debugger on the next iteration.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
+#[non_exhaustive]
 pub enum DebuggerCommand {
     // Continue normal execution.
     Continue,
@@ -60,29 +55,31 @@ impl Default for DebuggerCommand {
 
 /// The debugger status.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
+#[non_exhaustive]
 pub enum DebuggerStatus {
+    // Script evaluation starts.
+    Init,
     // Stop at the next statement or expression.
     Next(bool, bool),
     // Run to the end of the current level of function call.
     FunctionExit(usize),
-}
-
-impl Default for DebuggerStatus {
-    #[inline(always)]
-    fn default() -> Self {
-        Self::CONTINUE
-    }
+    // Script evaluation ends.
+    Terminate,
 }
 
 impl DebuggerStatus {
     pub const CONTINUE: Self = Self::Next(false, false);
     pub const STEP: Self = Self::Next(true, true);
     pub const NEXT: Self = Self::Next(true, false);
+    pub const INTO: Self = Self::Next(false, true);
 }
 
 /// A event that triggers the debugger.
 #[derive(Debug, Clone, Copy)]
+#[non_exhaustive]
 pub enum DebuggerEvent<'a> {
+    // Script evaluation starts.
+    Start,
     // Break on next step.
     Step,
     // Break on break-point.
@@ -91,10 +88,13 @@ pub enum DebuggerEvent<'a> {
     FunctionExitWithValue(&'a Dynamic),
     // Return from a function with a value.
     FunctionExitWithError(&'a EvalAltResult),
+    // Script evaluation ends.
+    End,
 }
 
 /// A break-point for debugging.
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
+#[non_exhaustive]
 pub enum BreakPoint {
     /// Break at a particular position under a particular source.
     ///
@@ -160,10 +160,7 @@ impl fmt::Display for BreakPoint {
                     f,
                     "{} ({})",
                     fn_name,
-                    std::iter::repeat("_")
-                        .take(*args)
-                        .collect::<Vec<_>>()
-                        .join(", ")
+                    repeat("_").take(*args).collect::<Vec<_>>().join(", ")
                 )?;
                 if !*enabled {
                     f.write_str(" (disabled)")?;
@@ -252,8 +249,6 @@ impl fmt::Display for CallStackFrame {
 pub struct Debugger {
     /// The current status command.
     pub(crate) status: DebuggerStatus,
-    /// The current state.
-    state: Dynamic,
     /// The current set of break-points.
     break_points: Vec<BreakPoint>,
     /// The current function call stack.
@@ -261,37 +256,16 @@ pub struct Debugger {
 }
 
 impl Debugger {
-    /// Create a new [`Debugger`] based on an [`Engine`].
+    /// Create a new [`Debugger`].
     #[inline(always)]
     #[must_use]
-    pub fn new(engine: &Engine) -> Self {
+    pub fn new(status: DebuggerStatus) -> Self {
         Self {
-            status: if engine.debugger.is_some() {
-                DebuggerStatus::STEP
-            } else {
-                DebuggerStatus::CONTINUE
-            },
-            state: if let Some((ref init, ..)) = engine.debugger {
-                init()
-            } else {
-                Dynamic::UNIT
-            },
+            status,
             break_points: Vec::new(),
             call_stack: Vec::new(),
         }
     }
-    /// Get a reference to the current state.
-    #[inline(always)]
-    #[must_use]
-    pub fn state(&self) -> &Dynamic {
-        &self.state
-    }
-    /// Get a mutable reference to the current state.
-    #[inline(always)]
-    #[must_use]
-    pub fn state_mut(&mut self) -> &mut Dynamic {
-        &mut self.state
-    }
     /// Get the current call stack.
     #[inline(always)]
     #[must_use]
@@ -409,7 +383,6 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         node: impl Into<ASTNode<'a>>,
@@ -417,7 +390,7 @@ impl Engine {
     ) -> RhaiResultOf<()> {
         if self.debugger.is_some() {
             if let Some(cmd) =
-                self.run_debugger_with_reset_raw(scope, global, state, lib, this_ptr, node, level)?
+                self.run_debugger_with_reset_raw(scope, global, lib, this_ptr, node, level)?
             {
                 global.debugger.status = cmd;
             }
@@ -437,14 +410,13 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         node: impl Into<ASTNode<'a>>,
         level: usize,
     ) -> RhaiResultOf<Option<DebuggerStatus>> {
         if self.debugger.is_some() {
-            self.run_debugger_with_reset_raw(scope, global, state, lib, this_ptr, node, level)
+            self.run_debugger_with_reset_raw(scope, global, lib, this_ptr, node, level)
         } else {
             Ok(None)
         }
@@ -461,7 +433,6 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         node: impl Into<ASTNode<'a>>,
@@ -475,25 +446,30 @@ impl Engine {
             _ => (),
         }
 
-        let stop = match global.debugger.status {
-            DebuggerStatus::Next(false, false) => false,
-            DebuggerStatus::Next(true, false) => matches!(node, ASTNode::Stmt(..)),
-            DebuggerStatus::Next(false, true) => matches!(node, ASTNode::Expr(..)),
-            DebuggerStatus::Next(true, true) => true,
-            DebuggerStatus::FunctionExit(..) => false,
+        let event = match global.debugger.status {
+            DebuggerStatus::Init => Some(DebuggerEvent::Start),
+            DebuggerStatus::CONTINUE => None,
+            DebuggerStatus::NEXT if matches!(node, ASTNode::Stmt(..)) => Some(DebuggerEvent::Step),
+            DebuggerStatus::NEXT => None,
+            DebuggerStatus::INTO if matches!(node, ASTNode::Expr(..)) => Some(DebuggerEvent::Step),
+            DebuggerStatus::INTO => None,
+            DebuggerStatus::STEP => Some(DebuggerEvent::Step),
+            DebuggerStatus::FunctionExit(..) => None,
+            DebuggerStatus::Terminate => Some(DebuggerEvent::End),
         };
 
-        let event = if stop {
-            DebuggerEvent::Step
-        } else {
-            if let Some(bp) = global.debugger.is_break_point(&global.source, node) {
-                DebuggerEvent::BreakPoint(bp)
-            } else {
-                return Ok(None);
+        let event = match event {
+            Some(e) => e,
+            None => {
+                if let Some(bp) = global.debugger.is_break_point(&global.source, node) {
+                    DebuggerEvent::BreakPoint(bp)
+                } else {
+                    return Ok(None);
+                }
             }
         };
 
-        self.run_debugger_raw(scope, global, state, lib, this_ptr, node, event, level)
+        self.run_debugger_raw(scope, global, lib, this_ptr, node, event, level)
     }
     /// Run the debugger callback unconditionally.
     ///
@@ -507,7 +483,6 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         node: ASTNode<'a>,
@@ -521,18 +496,18 @@ impl Engine {
             Some(source.as_str())
         };
 
-        let mut context = crate::EvalContext {
+        let context = crate::EvalContext {
             engine: self,
             scope,
             global,
-            state,
+            caches: None,
             lib,
             this_ptr,
             level,
         };
 
         if let Some((.., ref on_debugger)) = self.debugger {
-            let command = on_debugger(&mut context, event, node, source, node.position())?;
+            let command = on_debugger(context, event, node, source, node.position())?;
 
             match command {
                 DebuggerCommand::Continue => {
@@ -555,12 +530,12 @@ impl Engine {
                     // Bump a level if it is a function call
                     let level = match node {
                         ASTNode::Expr(Expr::FnCall(..)) | ASTNode::Stmt(Stmt::FnCall(..)) => {
-                            context.call_level() + 1
+                            level + 1
                         }
                         ASTNode::Stmt(Stmt::Expr(e)) if matches!(e.as_ref(), Expr::FnCall(..)) => {
-                            context.call_level() + 1
+                            level + 1
                         }
-                        _ => context.call_level(),
+                        _ => level,
                     };
                     global.debugger.status = DebuggerStatus::FunctionExit(level);
                     Ok(None)
