@@ -1,13 +1,14 @@
+mod cache;
 mod chaining;
 mod data_check;
 mod debugger;
 mod eval_context;
-mod eval_state;
 mod expr;
 mod global_state;
 mod stmt;
 mod target;
 
+pub use cache::{Caches, FnResolutionCache, FnResolutionCacheEntry};
 #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
 pub use chaining::{ChainArgument, ChainType};
 #[cfg(feature = "debugging")]
@@ -16,7 +17,6 @@ pub use debugger::{
     OnDebuggerCallback, OnDebuggingInit,
 };
 pub use eval_context::EvalContext;
-pub use eval_state::EvalState;
 #[cfg(not(feature = "no_module"))]
 #[cfg(not(feature = "no_function"))]
 pub use global_state::GlobalConstants;
