@@ -1,6 +1,6 @@
 //! Module defining functions for evaluating a statement.
 
-use super::{EvalContext, EvalState, GlobalRuntimeState, Target};
+use super::{Caches, EvalContext, GlobalRuntimeState, Target};
 use crate::api::events::VarDefInfo;
 use crate::ast::{
     ASTFlags, BinaryExpr, Expr, Ident, OpAssignment, Stmt, SwitchCases, TryCatchBlock,
@@ -25,7 +25,7 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         statements: &[Stmt],
@@ -36,14 +36,14 @@ impl Engine {
             return Ok(Dynamic::UNIT);
         }
 
-        let orig_always_search_scope = state.always_search_scope;
+        let orig_always_search_scope = global.always_search_scope;
         let orig_scope_len = scope.len();
         #[cfg(not(feature = "no_module"))]
         let orig_imports_len = global.num_imports();
-        let orig_fn_resolution_caches_len = state.fn_resolution_caches_len();
+        let orig_fn_resolution_caches_len = caches.fn_resolution_caches_len();
 
         if restore_orig_state {
-            state.scope_level += 1;
+            global.scope_level += 1;
         }
 
         let mut result = Ok(Dynamic::UNIT);
@@ -55,7 +55,7 @@ impl Engine {
             result = self.eval_stmt(
                 scope,
                 global,
-                state,
+                caches,
                 lib,
                 this_ptr,
                 stmt,
@@ -76,48 +76,46 @@ impl Engine {
                     .skip(imports_len)
                     .any(|(.., m)| m.contains_indexed_global_functions())
                 {
-                    if state.fn_resolution_caches_len() > orig_fn_resolution_caches_len {
+                    if caches.fn_resolution_caches_len() > orig_fn_resolution_caches_len {
                         // When new module is imported with global functions and there is already
                         // a new cache, clear it - notice that this is expensive as all function
                         // resolutions must start again
-                        state.fn_resolution_cache_mut().clear();
+                        caches.fn_resolution_cache_mut().clear();
                     } else if restore_orig_state {
                         // When new module is imported with global functions, push a new cache
-                        state.push_fn_resolution_cache();
+                        caches.push_fn_resolution_cache();
                     } else {
                         // When the block is to be evaluated in-place, just clear the current cache
-                        state.fn_resolution_cache_mut().clear();
+                        caches.fn_resolution_cache_mut().clear();
                     }
                 }
             }
         }
 
         // If imports list is modified, pop the functions lookup cache
-        state.rewind_fn_resolution_caches(orig_fn_resolution_caches_len);
+        caches.rewind_fn_resolution_caches(orig_fn_resolution_caches_len);
 
         if restore_orig_state {
             scope.rewind(orig_scope_len);
-            state.scope_level -= 1;
+            global.scope_level -= 1;
             #[cfg(not(feature = "no_module"))]
             global.truncate_imports(orig_imports_len);
 
             // The impact of new local variables goes away at the end of a block
             // because any new variables introduced will go out of scope
-            state.always_search_scope = orig_always_search_scope;
+            global.always_search_scope = orig_always_search_scope;
         }
 
         result
     }
 
     /// Evaluate an op-assignment statement.
-    /// [`Position`] in [`EvalAltResult`] is [`NONE`][Position::NONE] and should be set afterwards.
     pub(crate) fn eval_op_assignment(
         &self,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
-        op_info: Option<OpAssignment>,
-        op_pos: Position,
+        op_info: OpAssignment,
         target: &mut Target,
         root: (&str, Position),
         new_val: Dynamic,
@@ -130,13 +128,15 @@ impl Engine {
 
         let mut new_val = new_val;
 
-        if let Some(OpAssignment {
-            hash_op_assign,
-            hash_op,
-            op_assign,
-            op,
-        }) = op_info
-        {
+        if op_info.is_op_assignment() {
+            let OpAssignment {
+                hash_op_assign,
+                hash_op,
+                op_assign,
+                op,
+                pos: op_pos,
+            } = op_info;
+
             let mut lock_guard;
             let lhs_ptr_inner;
 
@@ -157,7 +157,7 @@ impl Engine {
             let level = level + 1;
 
             match self.call_native_fn(
-                global, state, lib, op_assign, hash, args, true, true, op_pos, level,
+                global, caches, lib, op_assign, hash, args, true, true, op_pos, level,
             ) {
                 Ok(_) => {
                     #[cfg(not(feature = "unchecked"))]
@@ -166,9 +166,11 @@ impl Engine {
                 Err(err) if matches!(*err, ERR::ErrorFunctionNotFound(ref f, ..) if f.starts_with(op_assign)) =>
                 {
                     // Expand to `var = var op rhs`
-                    let (value, ..) = self.call_native_fn(
-                        global, state, lib, op, hash_op, args, true, false, op_pos, level,
-                    )?;
+                    let (value, ..) = self
+                        .call_native_fn(
+                            global, caches, lib, op, hash_op, args, true, false, op_pos, level,
+                        )
+                        .map_err(|err| err.fill_position(op_info.pos))?;
 
                     #[cfg(not(feature = "unchecked"))]
                     self.check_data_size(&value, root.1)?;
@@ -182,7 +184,9 @@ impl Engine {
             *target.as_mut() = new_val;
         }
 
-        target.propagate_changed_value()
+        target
+            .propagate_changed_value()
+            .map_err(|err| err.fill_position(op_info.pos))
     }
 
     /// Evaluate a statement.
@@ -197,7 +201,7 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         stmt: &Stmt,
@@ -206,7 +210,7 @@ impl Engine {
     ) -> RhaiResult {
         #[cfg(feature = "debugging")]
         let reset_debugger =
-            self.run_debugger_with_reset(scope, global, state, lib, this_ptr, stmt, level)?;
+            self.run_debugger_with_reset(scope, global, lib, this_ptr, stmt, level)?;
 
         // Coded this way for better branch prediction.
         // Popular branches are lifted out of the `match` statement into their own branches.
@@ -217,7 +221,7 @@ impl Engine {
             self.inc_operations(&mut global.num_operations, stmt.position())?;
 
             let result =
-                self.eval_fn_call_expr(scope, global, state, lib, this_ptr, x, x.pos, level);
+                self.eval_fn_call_expr(scope, global, caches, lib, this_ptr, x, x.pos, level);
 
             #[cfg(feature = "debugging")]
             global.debugger.reset_status(reset_debugger);
@@ -228,7 +232,7 @@ impl Engine {
         // Then assignments.
         // We shouldn't do this for too many variants because, soon or later, the added comparisons
         // will cost more than the mis-predicted `match` branch.
-        if let Stmt::Assignment(x, op_pos) = stmt {
+        if let Stmt::Assignment(x, ..) = stmt {
             #[cfg(not(feature = "unchecked"))]
             self.inc_operations(&mut global.num_operations, stmt.position())?;
 
@@ -236,12 +240,12 @@ impl Engine {
                 let (op_info, BinaryExpr { lhs, rhs }) = x.as_ref();
 
                 let rhs_result = self
-                    .eval_expr(scope, global, state, lib, this_ptr, rhs, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, rhs, level)
                     .map(Dynamic::flatten);
 
                 if let Ok(rhs_val) = rhs_result {
                     let search_result =
-                        self.search_namespace(scope, global, state, lib, this_ptr, lhs, level);
+                        self.search_namespace(scope, global, lib, this_ptr, lhs, level);
 
                     if let Ok(search_val) = search_result {
                         let (mut lhs_ptr, pos) = search_val;
@@ -261,9 +265,8 @@ impl Engine {
                         let lhs_ptr = &mut lhs_ptr;
 
                         self.eval_op_assignment(
-                            global, state, lib, *op_info, *op_pos, lhs_ptr, root, rhs_val, level,
+                            global, caches, lib, *op_info, lhs_ptr, root, rhs_val, level,
                         )
-                        .map_err(|err| err.fill_position(rhs.start_position()))
                         .map(|_| Dynamic::UNIT)
                     } else {
                         search_result.map(|_| Dynamic::UNIT)
@@ -275,11 +278,11 @@ impl Engine {
                 let (op_info, BinaryExpr { lhs, rhs }) = x.as_ref();
 
                 let rhs_result = self
-                    .eval_expr(scope, global, state, lib, this_ptr, rhs, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, rhs, level)
                     .map(Dynamic::flatten);
 
                 if let Ok(rhs_val) = rhs_result {
-                    let _new_val = Some(((rhs_val, rhs.start_position()), (*op_info, *op_pos)));
+                    let _new_val = Some((rhs_val, *op_info));
 
                     // Must be either `var[index] op= val` or `var.prop op= val`
                     match lhs {
@@ -291,14 +294,14 @@ impl Engine {
                         #[cfg(not(feature = "no_index"))]
                         Expr::Index(..) => self
                             .eval_dot_index_chain(
-                                scope, global, state, lib, this_ptr, lhs, level, _new_val,
+                                scope, global, caches, lib, this_ptr, lhs, level, _new_val,
                             )
                             .map(|_| Dynamic::UNIT),
                         // dot_lhs.dot_rhs op= rhs
                         #[cfg(not(feature = "no_object"))]
                         Expr::Dot(..) => self
                             .eval_dot_index_chain(
-                                scope, global, state, lib, this_ptr, lhs, level, _new_val,
+                                scope, global, caches, lib, this_ptr, lhs, level, _new_val,
                             )
                             .map(|_| Dynamic::UNIT),
                         _ => unreachable!("cannot assign to expression: {:?}", lhs),
@@ -323,21 +326,21 @@ impl Engine {
 
             // Expression as statement
             Stmt::Expr(expr) => self
-                .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                 .map(Dynamic::flatten),
 
             // Block scope
             Stmt::Block(statements, ..) if statements.is_empty() => Ok(Dynamic::UNIT),
-            Stmt::Block(statements, ..) => {
-                self.eval_stmt_block(scope, global, state, lib, this_ptr, statements, true, level)
-            }
+            Stmt::Block(statements, ..) => self.eval_stmt_block(
+                scope, global, caches, lib, this_ptr, statements, true, level,
+            ),
 
             // If statement
             Stmt::If(x, ..) => {
                 let (expr, if_block, else_block) = x.as_ref();
 
                 let guard_val = self
-                    .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                     .and_then(|v| {
                         v.as_bool().map_err(|typ| {
                             self.make_type_mismatch_err::<bool>(typ, expr.position())
@@ -348,7 +351,7 @@ impl Engine {
                     Ok(true) => {
                         if !if_block.is_empty() {
                             self.eval_stmt_block(
-                                scope, global, state, lib, this_ptr, if_block, true, level,
+                                scope, global, caches, lib, this_ptr, if_block, true, level,
                             )
                         } else {
                             Ok(Dynamic::UNIT)
@@ -357,7 +360,7 @@ impl Engine {
                     Ok(false) => {
                         if !else_block.is_empty() {
                             self.eval_stmt_block(
-                                scope, global, state, lib, this_ptr, else_block, true, level,
+                                scope, global, caches, lib, this_ptr, else_block, true, level,
                             )
                         } else {
                             Ok(Dynamic::UNIT)
@@ -378,7 +381,8 @@ impl Engine {
                     },
                 ) = x.as_ref();
 
-                let value_result = self.eval_expr(scope, global, state, lib, this_ptr, expr, level);
+                let value_result =
+                    self.eval_expr(scope, global, caches, lib, this_ptr, expr, level);
 
                 if let Ok(value) = value_result {
                     let stmt_block_result = if value.is_hashable() {
@@ -388,21 +392,16 @@ impl Engine {
 
                         // First check hashes
                         if let Some(case_block) = cases.get(&hash) {
-                            let cond_result = case_block
-                                .condition
-                                .as_ref()
-                                .map(|cond| {
-                                    self.eval_expr(scope, global, state, lib, this_ptr, cond, level)
-                                        .and_then(|v| {
-                                            v.as_bool().map_err(|typ| {
-                                                self.make_type_mismatch_err::<bool>(
-                                                    typ,
-                                                    cond.position(),
-                                                )
-                                            })
+                            let cond_result = match case_block.condition {
+                                Expr::BoolConstant(b, ..) => Ok(b),
+                                ref c => self
+                                    .eval_expr(scope, global, caches, lib, this_ptr, c, level)
+                                    .and_then(|v| {
+                                        v.as_bool().map_err(|typ| {
+                                            self.make_type_mismatch_err::<bool>(typ, c.position())
                                         })
-                                })
-                                .unwrap_or(Ok(true));
+                                    }),
+                            };
 
                             match cond_result {
                                 Ok(true) => Ok(Some(&case_block.statements)),
@@ -420,23 +419,19 @@ impl Engine {
                                         || (inclusive && (start..=end).contains(&value))
                                 })
                             {
-                                let cond_result = block
-                                    .condition
-                                    .as_ref()
-                                    .map(|cond| {
-                                        self.eval_expr(
-                                            scope, global, state, lib, this_ptr, cond, level,
-                                        )
+                                let cond_result = match block.condition {
+                                    Expr::BoolConstant(b, ..) => Ok(b),
+                                    ref c => self
+                                        .eval_expr(scope, global, caches, lib, this_ptr, c, level)
                                         .and_then(|v| {
                                             v.as_bool().map_err(|typ| {
                                                 self.make_type_mismatch_err::<bool>(
                                                     typ,
-                                                    cond.position(),
+                                                    c.position(),
                                                 )
                                             })
-                                        })
-                                    })
-                                    .unwrap_or(Ok(true));
+                                        }),
+                                };
 
                                 match cond_result {
                                     Ok(true) => result = Ok(Some(&block.statements)),
@@ -460,7 +455,7 @@ impl Engine {
                     if let Ok(Some(statements)) = stmt_block_result {
                         if !statements.is_empty() {
                             self.eval_stmt_block(
-                                scope, global, state, lib, this_ptr, statements, true, level,
+                                scope, global, caches, lib, this_ptr, statements, true, level,
                             )
                         } else {
                             Ok(Dynamic::UNIT)
@@ -469,7 +464,7 @@ impl Engine {
                         // Default match clause
                         if !def_case.is_empty() {
                             self.eval_stmt_block(
-                                scope, global, state, lib, this_ptr, def_case, true, level,
+                                scope, global, caches, lib, this_ptr, def_case, true, level,
                             )
                         } else {
                             Ok(Dynamic::UNIT)
@@ -488,7 +483,7 @@ impl Engine {
 
                 if !body.is_empty() {
                     match self
-                        .eval_stmt_block(scope, global, state, lib, this_ptr, body, true, level)
+                        .eval_stmt_block(scope, global, caches, lib, this_ptr, body, true, level)
                     {
                         Ok(_) => (),
                         Err(err) => match *err {
@@ -508,7 +503,7 @@ impl Engine {
                 let (expr, body) = x.as_ref();
 
                 let condition = self
-                    .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                     .and_then(|v| {
                         v.as_bool().map_err(|typ| {
                             self.make_type_mismatch_err::<bool>(typ, expr.position())
@@ -519,9 +514,9 @@ impl Engine {
                     Ok(false) => break Ok(Dynamic::UNIT),
                     Ok(true) if body.is_empty() => (),
                     Ok(true) => {
-                        match self
-                            .eval_stmt_block(scope, global, state, lib, this_ptr, body, true, level)
-                        {
+                        match self.eval_stmt_block(
+                            scope, global, caches, lib, this_ptr, body, true, level,
+                        ) {
                             Ok(_) => (),
                             Err(err) => match *err {
                                 ERR::LoopBreak(false, ..) => (),
@@ -541,7 +536,7 @@ impl Engine {
 
                 if !body.is_empty() {
                     match self
-                        .eval_stmt_block(scope, global, state, lib, this_ptr, body, true, level)
+                        .eval_stmt_block(scope, global, caches, lib, this_ptr, body, true, level)
                     {
                         Ok(_) => (),
                         Err(err) => match *err {
@@ -553,7 +548,7 @@ impl Engine {
                 }
 
                 let condition = self
-                    .eval_expr(scope, global, state, lib, this_ptr, &expr, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, &expr, level)
                     .and_then(|v| {
                         v.as_bool().map_err(|typ| {
                             self.make_type_mismatch_err::<bool>(typ, expr.position())
@@ -572,7 +567,7 @@ impl Engine {
                 let (var_name, counter, expr, statements) = x.as_ref();
 
                 let iter_result = self
-                    .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                     .map(Dynamic::flatten);
 
                 if let Ok(iter_obj) = iter_result {
@@ -671,7 +666,7 @@ impl Engine {
                             }
 
                             let result = self.eval_stmt_block(
-                                scope, global, state, lib, this_ptr, statements, true, level,
+                                scope, global, caches, lib, this_ptr, statements, true, level,
                             );
 
                             match result {
@@ -715,7 +710,7 @@ impl Engine {
                 } = x.as_ref();
 
                 let result = self
-                    .eval_stmt_block(scope, global, state, lib, this_ptr, try_block, true, level)
+                    .eval_stmt_block(scope, global, caches, lib, this_ptr, try_block, true, level)
                     .map(|_| Dynamic::UNIT);
 
                 match result {
@@ -767,7 +762,7 @@ impl Engine {
                         let result = self.eval_stmt_block(
                             scope,
                             global,
-                            state,
+                            caches,
                             lib,
                             this_ptr,
                             catch_block,
@@ -794,7 +789,7 @@ impl Engine {
 
             // Throw value
             Stmt::Return(Some(expr), options, pos) if options.contains(ASTFlags::BREAK) => self
-                .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                 .and_then(|v| Err(ERR::ErrorRuntime(v.flatten(), *pos).into())),
 
             // Empty throw
@@ -804,7 +799,7 @@ impl Engine {
 
             // Return value
             Stmt::Return(Some(expr), .., pos) => self
-                .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                 .and_then(|v| Err(ERR::Return(v.flatten(), *pos).into())),
 
             // Empty return
@@ -828,7 +823,7 @@ impl Engine {
                 // Check variable definition filter
                 let result = if let Some(ref filter) = self.def_var_filter {
                     let will_shadow = scope.contains(var_name);
-                    let nesting_level = state.scope_level;
+                    let nesting_level = global.scope_level;
                     let is_const = access == AccessMode::ReadOnly;
                     let info = VarDefInfo {
                         name: var_name,
@@ -840,13 +835,13 @@ impl Engine {
                         engine: self,
                         scope,
                         global,
-                        state,
+                        caches: None,
                         lib,
                         this_ptr,
                         level,
                     };
 
-                    match filter(true, info, &context) {
+                    match filter(true, info, context) {
                         Ok(true) => None,
                         Ok(false) => {
                             Some(Err(
@@ -864,7 +859,7 @@ impl Engine {
                 } else {
                     // Evaluate initial value
                     let value_result = self
-                        .eval_expr(scope, global, state, lib, this_ptr, expr, level)
+                        .eval_expr(scope, global, caches, lib, this_ptr, expr, level)
                         .map(Dynamic::flatten);
 
                     if let Ok(mut value) = value_result {
@@ -872,7 +867,7 @@ impl Engine {
                             // Put global constants into global module
                             #[cfg(not(feature = "no_function"))]
                             #[cfg(not(feature = "no_module"))]
-                            if state.scope_level == 0
+                            if global.scope_level == 0
                                 && access == AccessMode::ReadOnly
                                 && lib.iter().any(|&m| !m.is_empty())
                             {
@@ -927,7 +922,7 @@ impl Engine {
                 }
 
                 let path_result = self
-                    .eval_expr(scope, global, state, lib, this_ptr, &expr, level)
+                    .eval_expr(scope, global, caches, lib, this_ptr, &expr, level)
                     .and_then(|v| {
                         let typ = v.type_name();
                         v.try_cast::<crate::ImmutableString>().ok_or_else(|| {
