@@ -3,7 +3,7 @@
 
 use super::call::FnCallArgs;
 use crate::ast::ScriptFnDef;
-use crate::eval::{EvalState, GlobalRuntimeState};
+use crate::eval::{Caches, GlobalRuntimeState};
 use crate::{Dynamic, Engine, Module, Position, RhaiError, RhaiResult, Scope, ERR};
 use std::mem;
 #[cfg(feature = "no_std")]
@@ -26,7 +26,7 @@ impl Engine {
         &self,
         scope: &mut Scope,
         global: &mut GlobalRuntimeState,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
         fn_def: &ScriptFnDef,
@@ -109,7 +109,7 @@ impl Engine {
         }
 
         // Merge in encapsulated environment, if any
-        let orig_fn_resolution_caches_len = state.fn_resolution_caches_len();
+        let orig_fn_resolution_caches_len = caches.fn_resolution_caches_len();
 
         #[cfg(not(feature = "no_module"))]
         let mut lib_merged = crate::StaticVec::with_capacity(lib.len() + 1);
@@ -121,14 +121,16 @@ impl Engine {
             ref constants,
         }) = fn_def.environ
         {
-            for (n, m) in imports.iter().cloned() {
-                global.push_import(n, m)
-            }
+            imports
+                .iter()
+                .cloned()
+                .for_each(|(n, m)| global.push_import(n, m));
+
             (
                 if fn_lib.is_empty() {
                     lib
                 } else {
-                    state.push_fn_resolution_cache();
+                    caches.push_fn_resolution_cache();
                     lib_merged.push(fn_lib.as_ref());
                     lib_merged.extend(lib.iter().cloned());
                     &lib_merged
@@ -142,7 +144,7 @@ impl Engine {
         #[cfg(feature = "debugging")]
         {
             let node = crate::ast::Stmt::Noop(fn_def.body.position());
-            self.run_debugger(scope, global, state, lib, this_ptr, &node, level)?;
+            self.run_debugger(scope, global, lib, this_ptr, &node, level)?;
         }
 
         // Evaluate the function
@@ -150,7 +152,7 @@ impl Engine {
             .eval_stmt_block(
                 scope,
                 global,
-                state,
+                caches,
                 lib,
                 this_ptr,
                 &fn_def.body,
@@ -167,7 +169,6 @@ impl Engine {
                     } else {
                         format!("{} @ '{}' < {}", name, src, fn_def.name)
                     };
-
                     make_error(fn_name, fn_def, global, err, pos)
                 }
                 // System errors are passed straight-through
@@ -193,8 +194,7 @@ impl Engine {
                     Ok(ref r) => crate::eval::DebuggerEvent::FunctionExitWithValue(r),
                     Err(ref err) => crate::eval::DebuggerEvent::FunctionExitWithError(err),
                 };
-                match self.run_debugger_raw(scope, global, state, lib, this_ptr, node, event, level)
-                {
+                match self.run_debugger_raw(scope, global, lib, this_ptr, node, event, level) {
                     Ok(_) => (),
                     Err(err) => _result = Err(err),
                 }
@@ -221,7 +221,7 @@ impl Engine {
         }
 
         // Restore state
-        state.rewind_fn_resolution_caches(orig_fn_resolution_caches_len);
+        caches.rewind_fn_resolution_caches(orig_fn_resolution_caches_len);
 
         _result
     }
@@ -231,11 +231,11 @@ impl Engine {
     pub(crate) fn has_script_fn(
         &self,
         _global: Option<&GlobalRuntimeState>,
-        state: &mut EvalState,
+        caches: &mut Caches,
         lib: &[&Module],
         hash_script: u64,
     ) -> bool {
-        let cache = state.fn_resolution_cache_mut();
+        let cache = caches.fn_resolution_cache_mut();
 
         if let Some(result) = cache.get(&hash_script).map(|v| v.is_some()) {
             return result;
