@@ -23,6 +23,7 @@ use std::{
 
 /// A type representing the namespace of a function.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[non_exhaustive]
 pub enum FnNamespace {
     /// Module namespace only.
     ///
@@ -34,6 +35,7 @@ pub enum FnNamespace {
 
 /// A type containing all metadata for a registered function.
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
+#[non_exhaustive]
 pub struct FnMetadata {
     /// Function namespace.
     pub namespace: FnNamespace,
@@ -1742,9 +1744,12 @@ impl Module {
 
     /// Create a new [`Module`] by evaluating an [`AST`][crate::AST].
     ///
-    /// The entire [`AST`][crate::AST] is encapsulated into each function, allowing functions
-    /// to cross-call each other.  Functions in the global namespace, plus all functions
-    /// defined in the [`Module`], are _merged_ into a _unified_ namespace before each call.
+    /// The entire [`AST`][crate::AST] is encapsulated into each function, allowing functions to
+    /// cross-call each other.
+    ///
+    /// Functions in the global namespace, plus all functions defined in the [`Module`], are
+    /// _merged_ into a _unified_ namespace before each call.
+    ///
     /// Therefore, all functions will be found.
     ///
     /// # Example
@@ -1781,8 +1786,15 @@ impl Module {
     /// _merged_ into a _unified_ namespace before each call.
     ///
     /// Therefore, all functions will be found.
+    ///
+    /// # WARNING - Low Level API
+    ///
+    /// This function is very low level.
+    ///
+    /// In particular, the [`global`][crate::eval::GlobalRuntimeState] parameter allows the
+    /// entire calling environment to be encapsulated, including automatic global constants.
     #[cfg(not(feature = "no_module"))]
-    pub(crate) fn eval_ast_as_new_raw(
+    pub fn eval_ast_as_new_raw(
         engine: &crate::Engine,
         scope: crate::Scope,
         global: &mut crate::eval::GlobalRuntimeState,
