@@ -3,7 +3,7 @@
 
 use crate::ast::{ASTFlags, Expr, OpAssignment, Stmt, StmtBlock, StmtBlockContainer, SwitchCases};
 use crate::engine::{KEYWORD_DEBUG, KEYWORD_EVAL, KEYWORD_FN_PTR, KEYWORD_PRINT, KEYWORD_TYPE_OF};
-use crate::eval::{EvalState, GlobalRuntimeState};
+use crate::eval::{Caches, GlobalRuntimeState};
 use crate::func::builtin::get_builtin_binary_op_fn;
 use crate::func::hashing::get_hasher;
 use crate::tokenizer::{Span, Token};
@@ -23,6 +23,7 @@ use std::{
 
 /// Level of optimization performed.
 #[derive(Debug, Eq, PartialEq, Hash, Clone, Copy)]
+#[non_exhaustive]
 pub enum OptimizationLevel {
     /// No optimization performed.
     None,
@@ -51,6 +52,10 @@ struct OptimizerState<'a> {
     propagate_constants: bool,
     /// An [`Engine`] instance for eager function evaluation.
     engine: &'a Engine,
+    /// The global runtime state.
+    global: GlobalRuntimeState<'a>,
+    /// Function resolution caches.
+    caches: Caches,
     /// [Module][crate::Module] containing script-defined functions.
     #[cfg(not(feature = "no_function"))]
     lib: &'a [&'a crate::Module],
@@ -61,7 +66,7 @@ struct OptimizerState<'a> {
 impl<'a> OptimizerState<'a> {
     /// Create a new State.
     #[inline(always)]
-    pub const fn new(
+    pub fn new(
         engine: &'a Engine,
         #[cfg(not(feature = "no_function"))] lib: &'a [&'a crate::Module],
         optimization_level: OptimizationLevel,
@@ -71,6 +76,8 @@ impl<'a> OptimizerState<'a> {
             variables: StaticVec::new_const(),
             propagate_constants: true,
             engine,
+            global: GlobalRuntimeState::new(engine),
+            caches: Caches::new(),
             #[cfg(not(feature = "no_function"))]
             lib,
             optimization_level,
@@ -127,7 +134,7 @@ impl<'a> OptimizerState<'a> {
     /// Call a registered function
     #[inline]
     pub fn call_fn_with_constant_arguments(
-        &self,
+        &mut self,
         fn_name: &str,
         arg_values: &mut [Dynamic],
     ) -> Option<Dynamic> {
@@ -138,8 +145,8 @@ impl<'a> OptimizerState<'a> {
 
         self.engine
             .call_native_fn(
-                &mut GlobalRuntimeState::new(&self.engine),
-                &mut EvalState::new(),
+                &mut self.global,
+                &mut self.caches,
                 lib,
                 fn_name,
                 calc_fn_hash(&fn_name, arg_values.len()),
@@ -426,7 +433,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
     match stmt {
         // var = var op expr => var op= expr
         Stmt::Assignment(x, ..)
-            if x.0.is_none()
+            if !x.0.is_op_assignment()
                 && x.1.lhs.is_variable_access(true)
                 && matches!(&x.1.rhs, Expr::FnCall(x2, ..)
                         if Token::lookup_from_syntax(&x2.name).map(|t| t.has_op_assignment()).unwrap_or(false)
@@ -437,7 +444,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
             match x.1.rhs {
                 Expr::FnCall(ref mut x2, ..) => {
                     state.set_dirty();
-                    x.0 = Some(OpAssignment::new_from_base(&x2.name));
+                    x.0 = OpAssignment::new_op_assignment_from_base(&x2.name, x2.pos);
                     x.1.rhs = mem::take(&mut x2.args[1]);
                 }
                 ref expr => unreachable!("Expr::FnCall expected but gets {:?}", expr),
@@ -531,35 +538,38 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
 
             // First check hashes
             if let Some(block) = cases.get_mut(&hash) {
-                if let Some(mut condition) = mem::take(&mut block.condition) {
-                    // switch const { case if condition => stmt, _ => def } => if condition { stmt } else { def }
-                    optimize_expr(&mut condition, state, false);
-
-                    let def_stmt =
-                        optimize_stmt_block(mem::take(def_case), state, true, true, false);
-
-                    *stmt = Stmt::If(
-                        (
-                            condition,
+                match mem::take(&mut block.condition) {
+                    Expr::BoolConstant(true, ..) => {
+                        // Promote the matched case
+                        let statements = optimize_stmt_block(
                             mem::take(&mut block.statements),
-                            StmtBlock::new_with_span(
-                                def_stmt,
-                                def_case.span_or_else(*pos, Position::NONE),
-                            ),
-                        )
-                            .into(),
-                        match_expr.start_position(),
-                    );
-                } else {
-                    // Promote the matched case
-                    let statements = optimize_stmt_block(
-                        mem::take(&mut block.statements),
-                        state,
-                        true,
-                        true,
-                        false,
-                    );
-                    *stmt = (statements, block.statements.span()).into();
+                            state,
+                            true,
+                            true,
+                            false,
+                        );
+                        *stmt = (statements, block.statements.span()).into();
+                    }
+                    mut condition => {
+                        // switch const { case if condition => stmt, _ => def } => if condition { stmt } else { def }
+                        optimize_expr(&mut condition, state, false);
+
+                        let def_stmt =
+                            optimize_stmt_block(mem::take(def_case), state, true, true, false);
+
+                        *stmt = Stmt::If(
+                            (
+                                condition,
+                                mem::take(&mut block.statements),
+                                StmtBlock::new_with_span(
+                                    def_stmt,
+                                    def_case.span_or_else(*pos, Position::NONE),
+                                ),
+                            )
+                                .into(),
+                            match_expr.start_position(),
+                        );
+                    }
                 }
 
                 state.set_dirty();
@@ -571,7 +581,11 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
                 let value = value.as_int().expect("`INT`");
 
                 // Only one range or all ranges without conditions
-                if ranges.len() == 1 || ranges.iter().all(|(.., c)| !c.has_condition()) {
+                if ranges.len() == 1
+                    || ranges
+                        .iter()
+                        .all(|(.., c)| matches!(c.condition, Expr::BoolConstant(true, ..)))
+                {
                     for (.., block) in
                         ranges
                             .iter_mut()
@@ -580,30 +594,38 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
                                     || (inclusive && (start..=end).contains(&value))
                             })
                     {
-                        if let Some(mut condition) = mem::take(&mut block.condition) {
-                            // switch const { range if condition => stmt, _ => def } => if condition { stmt } else { def }
-                            optimize_expr(&mut condition, state, false);
+                        match mem::take(&mut block.condition) {
+                            Expr::BoolConstant(true, ..) => {
+                                // Promote the matched case
+                                let statements = mem::take(&mut *block.statements);
+                                let statements =
+                                    optimize_stmt_block(statements, state, true, true, false);
+                                *stmt = (statements, block.statements.span()).into();
+                            }
+                            mut condition => {
+                                // switch const { range if condition => stmt, _ => def } => if condition { stmt } else { def }
+                                optimize_expr(&mut condition, state, false);
 
-                            let def_stmt =
-                                optimize_stmt_block(mem::take(def_case), state, true, true, false);
-                            *stmt = Stmt::If(
-                                (
-                                    condition,
-                                    mem::take(&mut block.statements),
-                                    StmtBlock::new_with_span(
-                                        def_stmt,
-                                        def_case.span_or_else(*pos, Position::NONE),
-                                    ),
-                                )
-                                    .into(),
-                                match_expr.start_position(),
-                            );
-                        } else {
-                            // Promote the matched case
-                            let statements = mem::take(&mut *block.statements);
-                            let statements =
-                                optimize_stmt_block(statements, state, true, true, false);
-                            *stmt = (statements, block.statements.span()).into();
+                                let def_stmt = optimize_stmt_block(
+                                    mem::take(def_case),
+                                    state,
+                                    true,
+                                    true,
+                                    false,
+                                );
+                                *stmt = Stmt::If(
+                                    (
+                                        condition,
+                                        mem::take(&mut block.statements),
+                                        StmtBlock::new_with_span(
+                                            def_stmt,
+                                            def_case.span_or_else(*pos, Position::NONE),
+                                        ),
+                                    )
+                                        .into(),
+                                    match_expr.start_position(),
+                                );
+                            }
                         }
 
                         state.set_dirty();
@@ -632,12 +654,14 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
                         *block.statements =
                             optimize_stmt_block(statements, state, preserve_result, true, false);
 
-                        if let Some(mut condition) = mem::take(&mut block.condition) {
-                            optimize_expr(&mut condition, state, false);
-                            match condition {
-                                Expr::Unit(..) | Expr::BoolConstant(true, ..) => state.set_dirty(),
-                                _ => block.condition = Some(condition),
+                        optimize_expr(&mut block.condition, state, false);
+
+                        match block.condition {
+                            Expr::Unit(pos) => {
+                                block.condition = Expr::BoolConstant(true, pos);
+                                state.set_dirty()
                             }
+                            _ => (),
                         }
                     }
                     return;
@@ -669,18 +693,20 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
                 *block.statements =
                     optimize_stmt_block(statements, state, preserve_result, true, false);
 
-                if let Some(mut condition) = mem::take(&mut block.condition) {
-                    optimize_expr(&mut condition, state, false);
-                    match condition {
-                        Expr::Unit(..) | Expr::BoolConstant(true, ..) => state.set_dirty(),
-                        _ => block.condition = Some(condition),
+                optimize_expr(&mut block.condition, state, false);
+
+                match block.condition {
+                    Expr::Unit(pos) => {
+                        block.condition = Expr::BoolConstant(true, pos);
+                        state.set_dirty();
                     }
+                    _ => (),
                 }
             }
 
             // Remove false cases
             cases.retain(|_, block| match block.condition {
-                Some(Expr::BoolConstant(false, ..)) => {
+                Expr::BoolConstant(false, ..) => {
                     state.set_dirty();
                     false
                 }
@@ -693,18 +719,20 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut OptimizerState, preserve_result: b
                 *block.statements =
                     optimize_stmt_block(statements, state, preserve_result, true, false);
 
-                if let Some(mut condition) = mem::take(&mut block.condition) {
-                    optimize_expr(&mut condition, state, false);
-                    match condition {
-                        Expr::Unit(..) | Expr::BoolConstant(true, ..) => state.set_dirty(),
-                        _ => block.condition = Some(condition),
+                optimize_expr(&mut block.condition, state, false);
+
+                match block.condition {
+                    Expr::Unit(pos) => {
+                        block.condition = Expr::BoolConstant(true, pos);
+                        state.set_dirty();
                     }
+                    _ => (),
                 }
             }
 
             // Remove false ranges
             ranges.retain(|(.., block)| match block.condition {
-                Some(Expr::BoolConstant(false, ..)) => {
+                Expr::BoolConstant(false, ..) => {
                     state.set_dirty();
                     false
                 }
