@@ -190,7 +190,7 @@ macro_rules! reg_functions {
 
 def_package! {
     /// Basic arithmetic package.
-    crate::ArithmeticPackage => |lib| {
+    pub ArithmeticPackage(lib) {
         lib.standard = true;
 
         combine_with_exported_module!(lib, "int", int_functions);
