@@ -1,5 +1,4 @@
 #![cfg(not(feature = "no_index"))]
-#![allow(non_snake_case)]
 
 use crate::engine::OP_EQUALS;
 use crate::eval::{calc_index, calc_offset_len};
@@ -14,7 +13,7 @@ use std::{any::TypeId, cmp::Ordering, mem};
 
 def_package! {
     /// Package of basic array utilities.
-    crate::BasicArrayPackage => |lib| {
+    pub BasicArrayPackage(lib) {
         lib.standard = true;
 
         combine_with_exported_module!(lib, "array", array_functions);
@@ -53,7 +52,7 @@ pub mod array_functions {
             return Dynamic::UNIT;
         }
 
-        let (index, _) = calc_offset_len(array.len(), index, 0);
+        let (index, ..) = calc_offset_len(array.len(), index, 0);
 
         if index >= array.len() {
             Dynamic::UNIT
@@ -89,7 +88,7 @@ pub mod array_functions {
             return;
         }
 
-        let (index, _) = calc_offset_len(array.len(), index, 0);
+        let (index, ..) = calc_offset_len(array.len(), index, 0);
 
         if index < array.len() {
             array[index] = value;
@@ -183,7 +182,7 @@ pub mod array_functions {
             return;
         }
 
-        let (index, _) = calc_offset_len(array.len(), index, 0);
+        let (index, ..) = calc_offset_len(array.len(), index, 0);
 
         if index >= array.len() {
             array.push(item);
@@ -232,10 +231,11 @@ pub mod array_functions {
             }
 
             let check_sizes = match item.0 {
-                crate::types::dynamic::Union::Array(_, _, _)
-                | crate::types::dynamic::Union::Str(_, _, _) => true,
+                crate::types::dynamic::Union::Array(..) | crate::types::dynamic::Union::Str(..) => {
+                    true
+                }
                 #[cfg(not(feature = "no_object"))]
-                crate::types::dynamic::Union::Map(_, _, _) => true,
+                crate::types::dynamic::Union::Map(..) => true,
                 _ => false,
             };
 
@@ -652,7 +652,7 @@ pub mod array_functions {
                 mapper
                     .call_raw(&ctx, None, [item.clone()])
                     .or_else(|err| match *err {
-                        ERR::ErrorFunctionNotFound(fn_sig, _)
+                        ERR::ErrorFunctionNotFound(fn_sig, ..)
                             if fn_sig.starts_with(mapper.fn_name()) =>
                         {
                             mapper.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
@@ -741,7 +741,7 @@ pub mod array_functions {
             if filter
                 .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
@@ -829,7 +829,7 @@ pub mod array_functions {
             if ctx
                 .call_fn_raw(OP_EQUALS, true, false, &mut [item, &mut value.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(ref fn_sig, _) if fn_sig.starts_with(OP_EQUALS) => {
+                    ERR::ErrorFunctionNotFound(ref fn_sig, ..) if fn_sig.starts_with(OP_EQUALS) => {
                         if item.type_id() == value.type_id() {
                             // No default when comparing same type
                             Err(err)
@@ -915,13 +915,13 @@ pub mod array_functions {
             return Ok(-1);
         }
 
-        let (start, _) = calc_offset_len(array.len(), start, 0);
+        let (start, ..) = calc_offset_len(array.len(), start, 0);
 
         for (i, item) in array.iter_mut().enumerate().skip(start) {
             if ctx
                 .call_fn_raw(OP_EQUALS, true, false, &mut [item, &mut value.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(ref fn_sig, _) if fn_sig.starts_with(OP_EQUALS) => {
+                    ERR::ErrorFunctionNotFound(ref fn_sig, ..) if fn_sig.starts_with(OP_EQUALS) => {
                         if item.type_id() == value.type_id() {
                             // No default when comparing same type
                             Err(err)
@@ -1045,13 +1045,13 @@ pub mod array_functions {
             return Ok(-1);
         }
 
-        let (start, _) = calc_offset_len(array.len(), start, 0);
+        let (start, ..) = calc_offset_len(array.len(), start, 0);
 
         for (i, item) in array.iter().enumerate().skip(start) {
             if filter
                 .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
@@ -1150,7 +1150,7 @@ pub mod array_functions {
             if filter
                 .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
@@ -1237,7 +1237,7 @@ pub mod array_functions {
             if !filter
                 .call_raw(&ctx, None, [item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [item.clone(), (i as INT).into()])
@@ -1487,7 +1487,7 @@ pub mod array_functions {
             result = reducer
                 .call_raw(&ctx, None, [result.clone(), item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(reducer.fn_name()) =>
                     {
                         reducer.call_raw(&ctx, None, [result, item, (i as INT).into()])
@@ -1649,7 +1649,7 @@ pub mod array_functions {
             result = reducer
                 .call_raw(&ctx, None, [result.clone(), item.clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(reducer.fn_name()) =>
                     {
                         reducer.call_raw(&ctx, None, [result, item, ((len - 1 - i) as INT).into()])
@@ -1926,7 +1926,7 @@ pub mod array_functions {
             if filter
                 .call_raw(&ctx, None, [array[x].clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [array[x].clone(), (i as INT).into()])
@@ -2122,7 +2122,7 @@ pub mod array_functions {
             if !filter
                 .call_raw(&ctx, None, [array[x].clone()])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(fn_sig, _)
+                    ERR::ErrorFunctionNotFound(fn_sig, ..)
                         if fn_sig.starts_with(filter.fn_name()) =>
                     {
                         filter.call_raw(&ctx, None, [array[x].clone(), (i as INT).into()])
@@ -2312,7 +2312,7 @@ pub mod array_functions {
             if !ctx
                 .call_fn_raw(OP_EQUALS, true, false, &mut [a1, a2])
                 .or_else(|err| match *err {
-                    ERR::ErrorFunctionNotFound(ref fn_sig, _) if fn_sig.starts_with(OP_EQUALS) => {
+                    ERR::ErrorFunctionNotFound(ref fn_sig, ..) if fn_sig.starts_with(OP_EQUALS) => {
                         if a1.type_id() == a2.type_id() {
                             // No default when comparing same type
                             Err(err)
