@@ -349,6 +349,7 @@ impl fmt::Debug for Span {
 /// _(internals)_ A Rhai language token.
 /// Exported under the `internals` feature only.
 #[derive(Debug, PartialEq, Clone, Hash)]
+#[non_exhaustive]
 pub enum Token {
     /// An `INT` constant.
     IntegerConstant(INT),
@@ -382,6 +383,8 @@ pub enum Token {
     LeftBracket,
     /// `]`
     RightBracket,
+    /// `()`
+    Unit,
     /// `+`
     Plus,
     /// `+` (unary)
@@ -558,6 +561,7 @@ impl Token {
             RightParen => ")",
             LeftBracket => "[",
             RightBracket => "]",
+            Unit => "()",
             Plus => "+",
             UnaryPlus => "+",
             Minus => "-",
@@ -754,6 +758,7 @@ impl Token {
             ")" => RightParen,
             "[" => LeftBracket,
             "]" => RightBracket,
+            "()" => Unit,
             "+" => Plus,
             "-" => Minus,
             "*" => Multiply,
@@ -1702,6 +1707,12 @@ fn get_next_token_inner(
             ('{', ..) => return Some((Token::LeftBrace, start_pos)),
             ('}', ..) => return Some((Token::RightBrace, start_pos)),
 
+            // Unit
+            ('(', ')') => {
+                eat_next(stream, pos);
+                return Some((Token::Unit, start_pos));
+            }
+
             // Parentheses
             ('(', '*') => {
                 eat_next(stream, pos);
