@@ -64,6 +64,28 @@ fn test_for_loop() -> Result<(), Box<EvalAltResult>> {
         45
     );
 
+    assert_eq!(
+        engine.eval::<INT>(
+            "
+                let sum = 0;
+                for x in 1..10 { sum += x; }
+                sum
+            "
+        )?,
+        45
+    );
+
+    assert_eq!(
+        engine.eval::<INT>(
+            "
+                let sum = 0;
+                for x in 1..=10 { sum += x; }
+                sum
+            "
+        )?,
+        55
+    );
+
     assert_eq!(
         engine.eval::<INT>(
             "
@@ -247,18 +269,37 @@ fn test_for_overflow() -> Result<(), Box<EvalAltResult>> {
 fn test_for_string() -> Result<(), Box<EvalAltResult>> {
     let engine = Engine::new();
 
-    let script = r#"
-        let s = "hello";
-        let sum = 0;
+    assert_eq!(
+        engine.eval::<INT>(
+            r#"
+                let s = "hello";
+                let sum = 0;
 
-        for ch in chars(s) {
-            sum += to_int(ch);
-        }
+                for ch in chars(s) {
+                    sum += to_int(ch);
+                }
 
-        sum
-    "#;
+                sum
+            "#
+        )?,
+        532
+    );
 
-    assert_eq!(engine.eval::<INT>(script)?, 532);
+    assert_eq!(
+        engine.eval::<INT>(
+            r#"
+                let s = "hello";
+                let sum = 0;
+
+                for ch in chars(s, 2..=3) {
+                    sum += to_int(ch);
+                }
+
+                sum
+            "#
+        )?,
+        216
+    );
 
     Ok(())
 }
@@ -330,3 +371,32 @@ fn test_for_module_iterator() -> Result<(), Box<EvalAltResult>> {
 
     Ok(())
 }
+
+#[test]
+#[cfg(not(feature = "no_index"))]
+#[cfg(not(feature = "no_closure"))]
+fn test_for_capture() -> Result<(), Box<EvalAltResult>> {
+    let engine = Engine::new();
+
+    assert_eq!(
+        engine.eval::<INT>(
+            "
+                let a = [];
+
+                for (x, i) in 100..110 {
+                    a += || i + x;
+                }
+
+                let sum = 0;
+
+                for fp in a {
+                    sum += call(fp);
+                }
+
+                sum
+            "
+        )?,
+        1180
+    );
+    Ok(())
+}
