@@ -1,6 +1,6 @@
 #![cfg(not(feature = "no_optimize"))]
 
-use rhai::{Engine, EvalAltResult, OptimizationLevel, Scope, INT};
+use rhai::{Engine, EvalAltResult, Module, OptimizationLevel, Scope, INT};
 
 #[test]
 fn test_optimizer() -> Result<(), Box<EvalAltResult>> {
@@ -74,38 +74,46 @@ fn test_optimizer_run() -> Result<(), Box<EvalAltResult>> {
 #[test]
 fn test_optimizer_parse() -> Result<(), Box<EvalAltResult>> {
     let mut engine = Engine::new();
+
     engine.set_optimization_level(OptimizationLevel::Simple);
 
     let ast = engine.compile("{ const DECISION = false; if DECISION { 42 } else { 123 } }")?;
 
-    assert_eq!(
-        format!("{:?}", ast),
-        "AST { source: \"\", body: Block[Expr(123 @ 1:53)], functions: Module, resolver: None }"
-    );
+    assert_eq!(format!("{:?}", ast), "AST { body: [Expr(123 @ 1:53)] }");
 
     let ast = engine.compile("const DECISION = false; if DECISION { 42 } else { 123 }")?;
 
     assert_eq!(
         format!("{:?}", ast),
-        r#"AST { source: "", body: Block[Var(false @ 1:18, "DECISION" @ 1:7, (Constant), 1:1), Expr(123 @ 1:51)], functions: Module, resolver: None }"#
+        r#"AST { body: [Var(("DECISION" @ 1:7, false @ 1:18, None), CONSTANT, 1:1), Expr(123 @ 1:51)] }"#
     );
 
     let ast = engine.compile("if 1 == 2 { 42 }")?;
 
-    assert_eq!(
-        format!("{:?}", ast),
-        "AST { source: \"\", body: Block[], functions: Module, resolver: None }"
-    );
+    assert_eq!(format!("{:?}", ast), "AST { body: [] }");
 
     engine.set_optimization_level(OptimizationLevel::Full);
 
     let ast = engine.compile("abs(-42)")?;
 
+    assert_eq!(format!("{:?}", ast), "AST { body: [Expr(42 @ 1:1)] }");
+
+    let ast = engine.compile("NUMBER")?;
+
     assert_eq!(
         format!("{:?}", ast),
-        "AST { source: \"\", body: Block[Expr(42 @ 1:1)], functions: Module, resolver: None }"
+        "AST { body: [Expr(Variable(NUMBER) @ 1:1)] }"
     );
 
+    let mut module = Module::new();
+    module.set_var("NUMBER", 42 as INT);
+
+    engine.register_global_module(module.into());
+
+    let ast = engine.compile("NUMBER")?;
+
+    assert_eq!(format!("{:?}", ast), "AST { body: [Expr(42 @ 1:1)] }");
+
     Ok(())
 }
 
