@@ -85,6 +85,8 @@ fn test_plugins_package() -> Result<(), Box<EvalAltResult>> {
 
     reg_functions!(engine += greet::single(INT, bool, char));
 
+    assert_eq!(engine.eval::<INT>("MYSTIC_NUMBER")?, 42);
+
     #[cfg(not(feature = "no_object"))]
     {
         assert_eq!(engine.eval::<INT>("let a = [1, 2, 3]; a.foo")?, 1);
@@ -93,7 +95,7 @@ fn test_plugins_package() -> Result<(), Box<EvalAltResult>> {
 
         assert!(
             matches!(*engine.run("const A = [1, 2, 3]; A.test(42);").expect_err("should error"),
-            EvalAltResult::ErrorAssignmentToConstant(x, _) if x == "array")
+            EvalAltResult::ErrorAssignmentToConstant(x, ..) if x == "array")
         )
     }
 
